==> super_scad_polygon-0.9.4-py3-none-any.whl: super_scad_polygon/SmoothPolygonMixin.py <==
from abc import ABC
from typing import List

from super_scad.d2.helper.PolygonSideExtender import PolygonSideExtender
from super_scad.d2.PolygonMixin import PolygonMixin
from super_scad.scad.Context import Context
from super_scad.scad.ScadWidget import ScadWidget
from super_scad_smooth_profile.RoughFactory import RoughFactory
from super_scad_smooth_profile.SmoothProfileFactory import SmoothProfileFactory
from super_scad_smooth_profile.SmoothProfileParams import SmoothProfileParams

from super_scad_polygon.helper.SmoothPolygonSideExtender import SmoothPolygonSideExtender


# class SmoothPolygonMixin(PolygonMixin, ScadWidget, ABC):
class SmoothPolygonMixin(ABC):
    """
    A widget for polygons with smooth corners.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, *, profile_factories: SmoothProfileFactory | List[SmoothProfileFactory] | None):
        """
        Object constructor.

        :param profile_factories: The profile factories to be applied at nodes of the polygon. When a single profile
                                  factory is given, this profile will be applied at all nodes.
        """
        self._profile_factories = profile_factories
        """
        The profile factories to be applied at nodes of the polygon.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def profile_factories(self) -> List[SmoothProfileFactory]:
        """
        Returns the list of smooth profile factories.
        """
        if isinstance(self._profile_factories, SmoothProfileFactory):
            self._profile_factories = [self._profile_factories for _ in range(self.sides)]

        elif isinstance(self._profile_factories, List):
            if len(self._profile_factories) < self.sides:
                self._profile_factories += [RoughFactory() for _ in range(len(self._profile_factories), self.sides)]

        elif self._profile_factories is None:
            self._profile_factories = [RoughFactory() for _ in range(self.sides)]

        else:
            raise ValueError(f'Parameter profile_factories SmoothProfileFactory, '
                             f', a list of SmoothProfileFactory or None, got {type(self._profile_factories)}')

        return self._profile_factories

    # ------------------------------------------------------------------------------------------------------------------
    def build(self, context: Context) -> ScadWidget:
        """
        Builds a SuperSCAD widget.

        :param context: The build context.
        """
        polygon = PolygonMixin.build(self, context)

        nodes = self.nodes
        inner_angles = self.inner_angles(context)
        normal_angles = self.normal_angles(context)
        extend_sides_by_eps = self.extend_sides_by_eps
        profile_factories = self.profile_factories
        n = len(nodes)
        for index in range(n):
            extend_side_by_eps1 = (index - 1) % n in extend_sides_by_eps
            extend_side_by_eps2 = index in extend_sides_by_eps

            params = SmoothProfileParams(inner_angle=inner_angles[index],
                                         normal_angle=normal_angles[index],
                                         position=nodes[index],
                                         side1_is_extended_by_eps=extend_side_by_eps1,
                                         side2_is_extended_by_eps=extend_side_by_eps2)
            polygon = profile_factories[index].create_smooth_profile(params=params, child=polygon)

        return polygon

    # ------------------------------------------------------------------------------------------------------------------
    def _create_polygon_side_extender(self) -> PolygonSideExtender:
        """
        Returns a polygon side extender that extends this polygon.
        """
        return SmoothPolygonSideExtender(self.profile_factories)

# ----------------------------------------------------------------------------------------------------------------------

==> super_scad_polygon-0.9.5-py3-none-any.whl: super_scad_polygon/SmoothPolygonMixin.py <==
from abc import ABC
from typing import List

from super_scad.d2.helper.PolygonSideExtender import PolygonSideExtender
from super_scad.d2.PolygonMixin import PolygonMixin
from super_scad.scad.Context import Context
from super_scad.scad.ScadWidget import ScadWidget
from super_scad_smooth_profile.Rough import Rough
from super_scad_smooth_profile.SmoothProfile import SmoothProfile
from super_scad_smooth_profile.SmoothProfileParams import SmoothProfileParams

from super_scad_polygon.helper.SmoothPolygonSideExtender import SmoothPolygonSideExtender


# class SmoothPolygonMixin(PolygonMixin, ScadWidget, ABC):
class SmoothPolygonMixin(ABC):
    """
    A widget for polygons with smooth corners.
    """

    # ------------------------------------------------------------------------------------------------------------------
    def __init__(self, *, profiles: SmoothProfile | List[SmoothProfile] | None):
        """
        Object constructor.

        :param profiles: The profile to be applied at nodes of the polygon. When a single profile is given, this profile
                         will be applied at all nodes.
        """
        self._profiles = profiles
        """
        The profile to be applied at nodes of the polygon.
        """

    # ------------------------------------------------------------------------------------------------------------------
    @property
    def profiles(self) -> List[SmoothProfile]:
        """
        Returns the list of smoothing profiles.
        """
        if isinstance(self._profiles, SmoothProfile):
            self._profiles = [self._profiles for _ in range(self.sides)]

        elif isinstance(self._profiles, List):
            if len(self._profiles) < self.sides:
                self._profiles += [Rough() for _ in range(len(self._profiles), self.sides)]

        elif self._profiles is None:
            self._profiles = [Rough() for _ in range(self.sides)]

        else:
            raise ValueError(f'Parameter profiles must be a SmoothProfile, '
                             f', a list of SmoothProfile or None, got {type(self._profiles)}')

        return self._profiles

    # ------------------------------------------------------------------------------------------------------------------
    def build(self, context: Context) -> ScadWidget:
        """
        Builds a SuperSCAD widget.

        :param context: The build context.
        """
        polygon = PolygonMixin.build(self, context)

        nodes = self.nodes
        inner_angles = self.inner_angles(context)
        normal_angles = self.normal_angles(context)
        extend_sides_by_eps = self.extend_sides_by_eps
        profiles = self.profiles
        n = len(nodes)
        for index in range(n):
            extend_side_by_eps1 = (index - 1) % n in extend_sides_by_eps
            extend_side_by_eps2 = index in extend_sides_by_eps

            params = SmoothProfileParams(inner_angle=inner_angles[index],
                                         normal_angle=normal_angles[index],
                                         position=nodes[index],
                                         side1_is_extended_by_eps=extend_side_by_eps1,
                                         side2_is_extended_by_eps=extend_side_by_eps2)
            polygon = profiles[index].create_smooth_profile(params=params, child=polygon)

        return polygon

    # ------------------------------------------------------------------------------------------------------------------
    def _create_polygon_side_extender(self) -> PolygonSideExtender:
        """
        Returns a polygon side extender that extends this polygon.
        """
        return SmoothPolygonSideExtender(self.profiles)

# ----------------------------------------------------------------------------------------------------------------------
